@@ -1,12 +1,13 @@
 # entry point for deepforest model
 import os
 import pandas as pd
-from skimage import io
+from PIL import Image
 import torch
 
 import pytorch_lightning as pl
 from torch import optim
 import matplotlib
+import numpy as np
 
 from deepforest import utilities
 from deepforest import dataset
@@ -14,6 +15,7 @@ from deepforest import get_data
 from deepforest import model
 from deepforest import predict
 from deepforest import visualize
+from deepforest import preprocess
 from deepforest import evaluate as evaluate_iou
 from pytorch_lightning.callbacks import LearningRateMonitor
 
@@ -26,7 +28,7 @@ class deepforest(pl.LightningModule):
         Args:
             num_classes (int): number of classes in the model
         Returns:
-            self: a deepforest pytorch ligthning module
+            self: a deepforest pytorch lightning module
         """
         super().__init__()
         
@@ -56,7 +58,7 @@ class deepforest(pl.LightningModule):
 
         self.num_classes = num_classes
         self.create_model()
-        
+                
         #Label encoder and decoder
         if not len(label_dict) == num_classes:
             raise ValueError(
@@ -67,7 +69,10 @@ class deepforest(pl.LightningModule):
         self.numeric_to_label_dict = {v: k for k, v in label_dict.items()}
         
         #Add user supplied transforms
-        self.transforms = dataset.get_transform
+        if transforms is None:
+            self.transforms = dataset.get_transform
+        else:
+            self.transforms = transforms
         
     def use_release(self):
         """Use the latest DeepForest model release from github and load model.
@@ -90,7 +95,7 @@ class deepforest(pl.LightningModule):
                                         self.config["score_thresh"])
 
     def create_trainer(self, logger=None, callbacks=[], **kwargs):
-        """Create a pytorch ligthning training by reading config files
+        """Create a pytorch lightning training by reading config files
         Args:
             callbacks (list): a list of pytorch-lightning callback classes
         """
@@ -105,7 +110,7 @@ class deepforest(pl.LightningModule):
                                   max_epochs=self.config["train"]["epochs"],
                                   gpus=self.config["gpus"],
                                   checkpoint_callback=False,
-                                  distributed_backend=self.config["distributed_backend"],
+                                  accelerator=self.config["distributed_backend"],
                                   fast_dev_run=self.config["train"]["fast_dev_run"],
                                   callbacks=callbacks,
                                   **kwargs)
@@ -183,10 +188,10 @@ class deepforest(pl.LightningModule):
         return loader
 
     def predict_image(self, image=None, path=None, return_plot=False):
-        """Predict an image with a deepforest model
-
+        """Predict a single image with a deepforest model
+                
         Args:
-            image: a numpy array of a RGB image ranged from 0-255
+            image: a float32 numpy array of a RGB with channels last format
             path: optional path to read image from disk instead of passing image arg
             return_plot: Return image with plotted detections
         Returns:
@@ -197,14 +202,21 @@ class deepforest(pl.LightningModule):
             raise ValueError(
                 "Path provided instead of image. If you want to predict an image from disk, is path ="
             )
+        
 
         if path:
             if not isinstance(path, str):
                 raise ValueError("Path expects a string path to image on disk")
-            image = io.imread(path)
-
+            image = np.array(Image.open(path).convert("RGB")).astype("float32")
+        
+        #sanity checks on input images
+        if not type(image) == np.ndarray:
+            raise TypeError("Input image is of type {}, expected numpy, if reading from PIL, wrap in np.array(image).astype(float32)".format(type(image)))
+        if not image.dtype =="float32":
+            raise ValueError("image must be float32 dtype, found {}, use image.astype('float32')".format(image.dtype))
+                        
             # Load on GPU is available
-        if torch.cuda.is_available():
+        if self.current_device.type == "cuda":
             self.model = self.model.to("cuda")
 
         self.model.eval()
@@ -236,6 +248,7 @@ class deepforest(pl.LightningModule):
         Returns:
             df: pandas dataframe with bounding boxes, label and scores for each image in the csv file
         """
+        self.model = self.model.to(self.current_device)
         self.model.eval()
         result = predict.predict_file(model=self.model,
                                       csv_file=csv_file,
@@ -314,7 +327,12 @@ class deepforest(pl.LightningModule):
     def training_step(self, batch, batch_idx):
         """Train on a loaded dataset
         """
-        path, images, targets = batch
+        #allow for empty data if data augmentation is generated
+        try:
+            path, images, targets = batch
+        except:
+            print("Empty batch encountered, skipping")
+            return None
 
         loss_dict = self.model.forward(images, targets)
 
@@ -327,7 +345,11 @@ class deepforest(pl.LightningModule):
         """Train on a loaded dataset
 
         """
-        path, images, targets = batch
+        try:
+            path, images, targets = batch
+        except:
+            print("Empty batch encountered, skipping")
+            return None
 
         self.model.train()
         loss_dict = self.model.forward(images, targets)
